@@ -67,33 +67,26 @@ Author: putkoff
 Version: 1.0
 """
 # -*- coding: UTF-8 -*-
-import requests
-import os
 # Google Chrome Driver
-from selenium import webdriver
+import os
 import ssl
 import re
 import yt_dlp
-import threading
-import requests
-from requests.adapters import HTTPAdapter
-from typing import Optional, List,Union
-from requests.packages.urllib3.poolmanager import PoolManager
-from requests.packages.urllib3.util import ssl_
-from urllib.parse import urlparse
-import logging
-from urllib.parse import urljoin
-from bs4 import BeautifulSoup
-import xml.etree.ElementTree as ET
-from abstract_utilities.time_utils import get_time_stamp,get_sleep,sleep_count_down
-from abstract_utilities.string_clean import eatInner,eatAll,eatOuter
-from abstract_utilities.json_utils import convert_to_json
 import socket
 import shutil
+import logging
+import requests
+from bs4 import BeautifulSoup
+from selenium import webdriver
+import xml.etree.ElementTree as ET
+from typing import Optional,List,Union
+from requests.adapters import HTTPAdapter
+from urllib.parse import urlparse, urljoin
+from requests.packages.urllib3.util import ssl_
+from requests.packages.urllib3.poolmanager import PoolManager
+from abstract_utilities import get_time_stamp,get_sleep,sleep_count_down,eatInner,eatAll,eatOuter,ThreadManager
 logging.basicConfig(level=logging.INFO)
 
-
-
 class DynamicRateLimiterManager:
     def __init__(self):
         # Key: Service Name, Value: DynamicRateLimiter instance
@@ -261,11 +254,12 @@ class CipherManagerSingleton:
         return CipherManagerSingleton._instance
 class SSLManager:
     def __init__(self, ciphers=None, ssl_options=None, certification=None):
-        self.ciphers = ciphers or "ECDHE-RSA-AES256-GCM-SHA384:ECDHE-ECDSA-AES256-GCM-SHA384:ECDHE-RSA-AES256-SHA384:ECDHE-ECDSA-AES256-SHA384:ECDHE-RSA-AES256-SHA:ECDHE-ECDSA-AES256-SHA:ECDHE-RSA-AES128-GCM-SHA256:ECDHE-RSA-AES128-SHA256:ECDHE-ECDSA-AES128-GCM-SHA256:ECDHE-ECDSA-AES128-SHA256:AES256-SHA:AES128-SHA"
-        self.ssl_options = ssl_options or ssl.OP_NO_TLSv1 | ssl.OP_NO_TLSv1_1 | ssl.OP_NO_COMPRESSION
+        self.ciphers = ciphers or CipherManager().ciphers_string
+        self.ssl_options = ssl_options or self.get_default_ssl_settings()
         self.certification = certification or ssl.CERT_REQUIRED
         self.ssl_context = self.get_context()
-
+    def get_default_ssl_settings(self):
+        return ssl.OP_NO_TLSv1 | ssl.OP_NO_TLSv1_1 | ssl.OP_NO_COMPRESSION
     def get_context(self):
         return ssl_.create_urllib3_context(ciphers=self.ciphers, cert_reqs=self.certification, options=self.ssl_options)
 
@@ -305,27 +299,41 @@ class TLSAdapterSingleton:
             TLSAdapterSingleton._instance = TLSAdapter(ciphers=ciphers, certification=certification, ssl_options=ssl_options)
         return TLSAdapterSingleton._instance
 class UserAgentManager:
-    def __init__(self, user_agent=None):
-        if user_agent == None:
-            user_agent = self.desktop_user_agents()[0]
-        self.user_agent = user_agent
-        self.user_agent=self.get_user_agent(self.user_agent)
+    def __init__(self, os=None, browser=None, version=None,user_agent=None):
+        self.os = os or 'Windows'
+        self.browser = browser or "Firefox"
+        self.version = version or '42.0'
+        self.user_agent = user_agent or self.get_user_agent()
+    
     @staticmethod
-    def desktop_user_agents() -> list:
-        return ['Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/89.0.4389.90 Safari/537.36','Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:87.0) Gecko/20100101 Firefox/87.0','Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/14.1 Safari/605.1.15','Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.124 Safari/537.36 Edg/91.0.864.59','Mozilla/5.0 (Windows NT 10.0; WOW64; Trident/7.0; rv:11.0) like Gecko','Opera/9.80 (Windows NT 6.0) Presto/2.12.388 Version/12.14']
-    @staticmethod
-    def big_user_agent_list(n=0):
-        from .big_user_agent_list import big_user_agent_list
-        return big_user_agent_list[n]
-    @staticmethod
-    def get_user_agent(user_agent: str = desktop_user_agents()[0]) -> dict:
-        if isinstance(user_agent,dict):
-            return user_agent
-        return {"user-agent": user_agent}
+    def user_agent_db():
+        from .big_user_agent_list import big_user_agent_dict
+        return big_user_agent_dict
+
+    def get_user_agent(self):
+        ua_db = self.user_agent_db()
+
+        if self.os and self.os in ua_db:
+            os_db = ua_db[self.os]
+        else:
+            os_db = random.choice(list(ua_db.values()))
+
+        if self.browser and self.browser in os_db:
+            browser_db = os_db[self.browser]
+        else:
+            browser_db = random.choice(list(os_db.values()))
+
+        if self.version and self.version in browser_db:
+            return browser_db[self.version]
+        else:
+            return random.choice(list(browser_db.values()))
+
+    def user_agent_header(self):
+        return {"user-agent": self.user_agent}
 class UserAgentManagerSingleton:
     _instance = None
     @staticmethod
-    def get_instance(user_agent=UserAgentManager.desktop_user_agents()[0]):
+    def get_instance(user_agent=UserAgentManager().get_user_agent()[0]):
         if UserAgentManagerSingleton._instance is None:
             UserAgentManagerSingleton._instance = UserAgentManager(user_agent=user_agent)
         elif UserAgentManagerSingleton._instance.user_agent != user_agent:
@@ -390,17 +398,7 @@ class MySocketClient():
         elif MySocketClientSingleton._instance.ip_address != ip_address or MySocketClientSingleton._instance.port != port or UrlManagerSingleton._instance.domain != domain:
             MySocketClientSingleton._instance = MySocketClient(ip_address=ip_address,port=port,domain=domain)
         return MySocketClient
-def safe_json_loads(data):
-    try:
-        return json.loads(data)
-    except json.JSONDecodeError:
-        return None
-def convert_to_json(obj):
-    if isinstance(obj, dict):
-        return obj
-    if isinstance(obj, str):
-        return safe_json_loads(obj)
-    return None
+
 class UrlManager:
     """
     UrlManager is a class for managing URLs, including cleaning, validating, and finding the correct version.
@@ -435,7 +433,7 @@ class UrlManager:
         - It also includes methods for joining and validating relative URLs.
     """
 
-    def __init__(self, url='www.example.com', session=requests):
+    def __init__(self, url=None, session=None):
         """
         Initialize a UrlManager instance.
 
@@ -443,25 +441,27 @@ class UrlManager:
             url (str or None): The URL to manage (default is None).
             session (requests.Session): A custom requests session (default is the requests module's session).
         """
-        self_url=url or ""
-        self.session=session
+        self._url=url or 'www.example.com'
+        self.url = url or 'www.example.com'
+        self.session= session or requests
         self.clean_urls = self.clean_url(url=url)
         self.url = self.get_correct_url(clean_urls=self.clean_urls)
-        self.protocol,self.domain,self.path,self.query=self.url_to_pieces(url=self.url)
+        url_pieces = self.url_to_pieces(url=self.url)
+        self.protocol,self.domain,self.path,self.query=url_pieces
         self.all_urls = []
     def url_to_pieces(self, url):
+        
         try:
             match = re.match(r'^(https?)?://?([^/]+)(/[^?]+)?(\?.+)?', url)
             if match:
                 protocol = match.group(1) if match.group(1) else None
-                domain = match.group(2)
+                domain = match.group(2) if match.group(1) else None
                 path = match.group(3) if match.group(3) else ""  # Handle None
                 query = match.group(4) if match.group(4) else ""  # Handle None
-                return protocol, domain, path, query
         except:
             print(f'the url {url} was not reachable')
-            return None
-        
+            protocol,domain,path,query=None,None,"",""
+        return protocol, domain, path, query
 
     def clean_url(self,url=None) -> list:
         """
@@ -470,6 +470,7 @@ class UrlManager:
         """
         if url == None:
             url=self.url
+        urls=[]
         if url:
             # Remove http:// or https:// prefix
             cleaned = url.replace("http://", "").replace("https://", "")
@@ -494,7 +495,7 @@ class UrlManager:
                     f"http://www.{cleaned}",
                 ])
 
-            return urls
+        return urls
 
     def get_correct_url(self,url=None,clean_urls=None) -> (str or None):
         """
@@ -508,6 +509,9 @@ class UrlManager:
         Returns:
             str: The correct version of the URL if found, or None if none of the variations are valid.
         """
+        if url==None and clean_urls != None:
+            if self.url:
+                url=self.url or clean_urls[0]
         if url!=None and clean_urls==None:
             clean_urls=self.clean_url(url)
         elif url==None and clean_urls==None:
@@ -656,10 +660,12 @@ class SafeRequest:
     """
     def __init__(self,
                  url=None,
+                 source_code=None,
                  url_manager=None,
                  network_manager=None,
                  user_agent_manager=None,
                  ssl_manager=None,
+                 ssl_options=None,
                  tls_adapter=None,
                  user_agent=None,
                  proxies=None,
@@ -679,22 +685,27 @@ class SafeRequest:
                  checkbox=None,
                  dropdown=None,
                  certification=None,
-                 ssl_options=None,
                  stream=False,
                  timeout = None,
                  last_request_time=None,
                  max_retries=None,
                  request_wait_limit=None):
+        self._url=url
+        self.url=url
         if url_manager == None:
-            url_manager = UrlManager(url=url)
+            url_manager = UrlManager(url=self.url)
         self.url_manager=url_manager
-        if network_manager == None:
-            network_manager=NetworkManager(user_agent_manager=user_agent_manager,ssl_manager=ssl_manager, tls_adapter=tls_adapter,user_agent=user_agent,proxies=proxies,cookies=cookies,ciphers=ciphers, certification=certification, ssl_options=ssl_options)
-        if user_agent_manager == None:
-            user_agent_manager = UserAgentManager(user_agent=user_agent)
-        self.user_agent_manager = user_agent_manager
-        self.user_agent= self.user_agent_manager.user_agent   
-        self.network_manager = network_manager
+        self._url_manager = self.url_manager
+        self.user_agent = user_agent
+        self.user_agent_manager = user_agent_manager or UserAgentManager(user_agent=self.user_agent)
+        self.headers= headers or self.user_agent_manager.user_agent or {'Accept': '*/*'}
+        self.user_agent= self.user_agent_manager.user_agent
+        self.ciphers=ciphers or CipherManager().ciphers_string
+        self.certification=certification
+        self.ssl_options=ssl_options
+        self.ssl_manager = ssl_manager or SSLManager(ciphers=self.ciphers, ssl_options=self.ssl_options, certification=self.certification)
+        self.tls_adapter=tls_adapter or  TLSAdapter(ssl_manager=self.ssl_manager,certification=self.certification,ssl_options=self.ssl_manager.ssl_options)
+        self.network_manager= network_manager or NetworkManager(user_agent_manager=self.user_agent_manager,ssl_manager=self.ssl_manager, tls_adapter=self.tls_adapter,user_agent=user_agent,proxies=proxies,cookies=cookies,ciphers=ciphers, certification=certification, ssl_options=ssl_options)
         self.stream=stream
         self.tls_adapter=self.network_manager.tls_adapter
         self.ciphers=self.network_manager.ciphers
@@ -714,7 +725,7 @@ class SafeRequest:
         self.login_user_agent=login_user_agent
         self.login_referer=login_referer
         self.protocol=protocol or 'https://'
-        self.headers=headers or self.user_agent or {'Accept': '*/*'}
+        
         self.stream=stream if isinstance(stream,bool) else False
         self.initialize_session()
         self.last_request_time=last_request_time
@@ -945,7 +956,7 @@ class SafeRequest:
 
     @url.setter
     def url(self, new_url):
-        self._url_manager.url = new_url
+        self._url = new_url
 class SafeRequestSingleton:
     _instance = None
     @staticmethod
@@ -1298,7 +1309,8 @@ class VideoDownloader:
             video_directory=os.path.join(os.getcwd(),'videos')
         if temp_directory == None:
             temp_directory=os.path.join(video_directory,'temp_files')
-        self.pause_event = threading.Event()
+        self.thread_manager = ThreadManager()
+        self.pause_event = self.thread_manager.add_thread('pause_event')
         self.link = link
         self.temp_directory = temp_directory
         self.video_directory = video_directory
@@ -1448,19 +1460,19 @@ class VideoDownloader:
             
     def monitor(self):
         while self.monitoring:
-            self.pause_event.wait(self.time_interval)# check every minute
+            self.thread_manager.wait(name='pause_event',n=self.time_interval)# check every minute
             if self.monitoring:
                 if 'eta' in self.status_dict:
                     if self.outlier_count>=3 and (self.status_dict['eta']/60)>10:
                         self.start()
 
     def start(self):
-        download_thread = threading.Thread(target=self.download)
-        monitor_thread = threading.Thread(target=self.monitor)
-        download_thread.start()
-        monitor_thread.start()
-        download_thread.join()
-        monitor_thread.join()
+        download_thread = self.thread_manager.add_thread(name='download_thread',target=self.download)
+        monitor_thread = self.thread_manager.add_thread(name='monitor_thread',target_function=self.monitor)
+        self.thread_manager.start(name='download_thread')
+        self.thread_manager.start(name='monitor_thread')
+        self.thread_manager.join(name='download_thread')
+        self.thread_manager.join(name='monitor_thread')
 class VideoDownloaderSingleton():
     _instance = None
     @staticmethod
